@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2020 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2022 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -62,7 +62,7 @@ hdeffminp <-
 
   ans <-
   new("vglmff",
-  blurb = if (multiple.responses) c("Multiple binomial model\n\n",
+  blurb = if (multiple.responses) c("Multiple responses binomial model\n\n",
          "Link:     ", namesof("mu[,j]", link, earg = earg), "\n",
          "Variance: mu[,j]*(1-mu[,j])") else
          c("Binomial model\n\n",
@@ -96,6 +96,7 @@ hdeffminp <-
   infos = eval(substitute(function(...) {
     list(M1 = 1,
          Q1 = 1,
+         dpqrfun = "binom",
          bred = .bred ,
          charfun = TRUE,
          expected = TRUE,
@@ -454,8 +455,7 @@ hdeffminp <-
     } else if ( .link == "clogloglink") {
       cbind(c(w) * (1.0 - mu.use) * (log1p(-mu.use))^2 / mu.use)
     } else {
-      cbind(c(w) * dtheta.deta(mu, link = .link ,
-                               earg = .earg )^2 / tmp100)
+      cbind(c(w) * dtheta.deta(mu, .link , earg = .earg )^2 / tmp100)
     }
     for (ii in 1:M) {
       index500 <- !is.finite(ned2ldprob2[, ii]) |
@@ -588,7 +588,7 @@ hdeffminp <-
     }
     misc$dispersion <- dpar
     misc$default.dispersion <- 0
-    misc$estimated.dispersion <- .estimated.dispersion
+    misc$estimated.dispersion <- ( .estimated.dispersion )
 
     misc$link <- rep_len( .link , M)
     names(misc$link) <- param.names("mu", M, skip1 = TRUE)
@@ -596,7 +596,7 @@ hdeffminp <-
     misc$earg <- vector("list", M)
     names(misc$earg) <- names(misc$link)
     for (ii in 1:M)
-      misc$earg[[ii]] <- .earg
+      misc$earg[[ii]] <- ( .earg )
 
     misc$expected <- TRUE
     misc$multipleResponses <- TRUE
@@ -632,6 +632,7 @@ hdeffminp <-
 
 
 
+
  inverse.gaussianff <- function(link = "natural.ig",
                                 dispersion = 0) {
   estimated.dispersion <- dispersion == 0
@@ -670,6 +671,8 @@ hdeffminp <-
   infos = eval(substitute(function(...) {
     list(M1 = 1,
          Q1 = 1,
+         expected = TRUE,
+         multipleResponses = TRUE,
          parameters.names = c("mu"),
          quasi.type = TRUE,
          dispersion = .dispersion )
@@ -715,7 +718,7 @@ hdeffminp <-
     }
     misc$dispersion <- dpar
     misc$default.dispersion <- 0
-    misc$estimated.dispersion <- .estimated.dispersion
+    misc$estimated.dispersion <- ( .estimated.dispersion )
 
     misc$link <- rep_len( .link , M)
     names(misc$link) <- param.names("mu", M, skip1 = TRUE)
@@ -723,10 +726,7 @@ hdeffminp <-
     misc$earg <- vector("list", M)
     names(misc$earg) <- names(misc$link)
     for (ii in 1:M)
-      misc$earg[[ii]] <- .earg
-
-    misc$expected <- TRUE
-    misc$multipleResponses <- TRUE
+      misc$earg[[ii]] <- ( .earg )
   }), list( .dispersion = dispersion,
             .estimated.dispersion = estimated.dispersion,
             .link = link, .earg = earg ))),
@@ -752,7 +752,7 @@ hdeffminp <-
     wz <- dmu.deta^2 * d2l.dmu2
     w.wz.merge(w = w, wz = wz, n = n, M = M, ndepy = ncoly)
   }), list( .link = link, .earg = earg ))))
-}
+}  # inverse.gaussianff
 
 
 
@@ -822,11 +822,12 @@ rinv.gaussian <- function(n, mu, lambda) {
 
 
 
- inv.gaussianff <- function(lmu = "loglink", llambda = "loglink",
-                            imethod = 1,  ilambda = NULL,
-                            parallel = FALSE,
-                            ishrinkage = 0.99,
-                            zero = NULL) {
+ inv.gaussianff <-
+  function(lmu = "loglink", llambda = "loglink",
+           imethod = 1,  ilambda = NULL,
+           parallel = FALSE,
+           ishrinkage = 0.99,
+           zero = NULL) {
 
 
   apply.parint <- FALSE
@@ -880,6 +881,7 @@ rinv.gaussian <- function(n, mu, lambda) {
   infos = eval(substitute(function(...) {
     list(M1 = 2,
          Q1 = 1,
+         dpqrfun = "inv.gaussian",
          imethod = .imethod ,
          parameters.names = c("mu", "lambda"),
          expected = TRUE,
@@ -1043,13 +1045,14 @@ rinv.gaussian <- function(n, mu, lambda) {
 
 
 
- poissonff <- function(link = "loglink",
-                       imu = NULL, imethod = 1,
-                       parallel = FALSE, zero = NULL,
-                       bred = FALSE,
-                       earg.link = FALSE,
-                       type.fitted = c("mean", "quantiles"),
-                       percentiles = c(25, 50, 75)) {
+ poissonff <-
+  function(link = "loglink",
+           imu = NULL, imethod = 1,
+           parallel = FALSE, zero = NULL,
+           bred = FALSE,
+           earg.link = FALSE,
+           type.fitted = c("mean", "quantiles"),
+           percentiles = c(25, 50, 75)) {
 
 
 
@@ -1113,6 +1116,7 @@ rinv.gaussian <- function(n, mu, lambda) {
   infos = eval(substitute(function(...) {
     list(M1 = 1,
          Q1 = 1,
+         dpqrfun = "pois",
          charfun = TRUE,
          expected = TRUE,
          hadof = TRUE,
@@ -1461,6 +1465,7 @@ if (FALSE)
 
 
 
+
  double.exppoisson <-
   function(lmean = "loglink",
            ldispersion = "logitlink",
@@ -1522,16 +1527,16 @@ if (FALSE)
         "mu"
     }
     predictors.names <-
-      c(namesof(dn2,          link = .lmean, earg = .emean, short = TRUE),
-        namesof("dispersion", link = .ldisp, earg = .edisp, short = TRUE))
+      c(namesof(dn2,          .lmean , earg = .emean, short = TRUE),
+        namesof("dispersion", .ldisp , earg = .edisp, short = TRUE))
 
     init.mu <- pmax(y, 1/8)
     tmp2 <- rep_len( .idisp , n)
 
     if (!length(etastart))
       etastart <-
-        cbind(theta2eta(init.mu, link = .lmean , earg = .emean ),
-              theta2eta(tmp2,    link = .ldisp , earg = .edisp ))
+        cbind(theta2eta(init.mu, .lmean , earg = .emean ),
+              theta2eta(tmp2,    .ldisp , earg = .edisp ))
   }), list( .lmean = lmean, .emean = emean,
             .ldisp = ldisp, .edisp = edisp,
             .idisp = idisp ))),
@@ -1550,10 +1555,8 @@ if (FALSE)
   loglikelihood = eval(substitute(
     function(mu, y, w, residuals = FALSE, eta, extra = NULL,
              summation = TRUE) {
-      lambda <- eta2theta(eta[, 1], link = .lmean,
-                          earg = .emean )
-      Disper <- eta2theta(eta[, 2], link = .ldisp,
-                          earg = .edisp )
+      lambda <- eta2theta(eta[, 1], .lmean , earg = .emean )
+      Disper <- eta2theta(eta[, 2], .ldisp , earg = .edisp )
     if (residuals) {
       stop("loglikelihood residuals not implemented yet")
     } else {
@@ -1569,8 +1572,8 @@ if (FALSE)
            .ldisp = ldisp, .edisp = edisp ))),
   vfamily = "double.exppoisson",
   validparams = eval(substitute(function(eta, y, extra = NULL) {
-    lambda <- eta2theta(eta[, 1], link = .lmean , earg = .emean )
-    Disper <- eta2theta(eta[, 2], link = .ldisp , earg = .edisp )
+    lambda <- eta2theta(eta[, 1], .lmean , earg = .emean )
+    Disper <- eta2theta(eta[, 2], .ldisp , earg = .edisp )
     okay1 <- all(is.finite(lambda)) && all(0 < lambda) &&
              all(is.finite(Disper)) && all(0 < Disper & Disper < 1)
     okay1
@@ -1579,16 +1582,14 @@ if (FALSE)
 
 
   deriv = eval(substitute(expression({
-    lambda <- eta2theta(eta[, 1], link = .lmean , earg = .emean )
-    Disper <- eta2theta(eta[, 2], link = .ldisp , earg = .edisp )
+    lambda <- eta2theta(eta[, 1], .lmean , earg = .emean )
+    Disper <- eta2theta(eta[, 2], .ldisp , earg = .edisp )
 
     dl.dlambda <- Disper * (y / lambda - 1)
     dl.dDisper <- y * log(lambda) + y - lambda + 0.5 / Disper
 
-    dlambda.deta <- dtheta.deta(theta = lambda, link = .lmean,
-                                earg = .emean)
-    dDisper.deta <- dtheta.deta(theta = Disper, link = .ldisp,
-                                earg = .edisp)
+    dlambda.deta <- dtheta.deta(theta = lambda, .lmean, earg = .emean)
+    dDisper.deta <- dtheta.deta(theta = Disper, .ldisp, earg = .edisp)
 
     c(w) * cbind(dl.dlambda * dlambda.deta,
                  dl.dDisper * dDisper.deta)
@@ -1603,7 +1604,7 @@ if (FALSE)
   }), list( .lmean = lmean, .emean = emean,
             .ldisp = ldisp,
             .edisp = edisp ))))
-}
+}  # double.exppoisson
 
 
 
@@ -1785,7 +1786,7 @@ if (FALSE)
     wz
   }), list( .lmean = lmean, .emean = emean,
             .ldisp = ldisp, .edisp = edisp ))))
-}
+}  # double.expbinomial
 
 
 
@@ -1796,8 +1797,9 @@ if (FALSE)
 
 
 
- augbinomial <- function(link = "logitlink", multiple.responses = FALSE,
-                         parallel = TRUE) {
+ augbinomial <-
+  function(link = "logitlink", multiple.responses = FALSE,
+           parallel = TRUE) {
 
   if (!is.logical(parallel) ||
       length(parallel) != 1 ||
@@ -2021,7 +2023,9 @@ if (FALSE)
     my.wk.wt <- my.wk.wt[, interleave.VGAM(M1 * NOS, M1 = M1)]
     my.wk.wt
   }), list( .link = link, .earg = earg))))
-}
+}  # augbinomial
+
+
 
 
 
